@@ -7,6 +7,7 @@ use skim::prelude::*;
 #[derive(Debug, Clone)]
 struct Item {
     text: String,
+    index: usize,
 }
 
 impl SkimItem for Item {
@@ -17,6 +18,14 @@ impl SkimItem for Item {
     fn preview(&self, _context: PreviewContext) -> ItemPreview {
         ItemPreview::Text(self.text.to_owned())
     }
+
+    fn get_index(&self) -> usize {
+        self.index
+    }
+
+    fn set_index(&mut self, index: usize) {
+        self.index = index
+    }
 }
 
 pub fn main() {
@@ -29,9 +38,21 @@ pub fn main() {
 
     let (tx, rx): (SkimItemSender, SkimItemReceiver) = unbounded();
 
-    tx.send(Arc::new(Item { text: "a".to_string() })).unwrap();
-    tx.send(Arc::new(Item { text: "b".to_string() })).unwrap();
-    tx.send(Arc::new(Item { text: "c".to_string() })).unwrap();
+    tx.send(Arc::new(Item {
+        text: "a".to_string(),
+        index: 0,
+    }))
+    .unwrap();
+    tx.send(Arc::new(Item {
+        text: "b".to_string(),
+        index: 1,
+    }))
+    .unwrap();
+    tx.send(Arc::new(Item {
+        text: "c".to_string(),
+        index: 2,
+    }))
+    .unwrap();
 
     drop(tx);
 
