@@ -22,6 +22,29 @@ use crate::{ItemPreview, PreviewContext, PreviewPosition, SkimItem};
 const TAB_STOP: usize = 8;
 const DELIMITER_STR: &str = r"[\t\n ]+";
 
+/// A wrapper around a thread-safe (Arc) to a function
+#[derive(Clone)]
+pub struct PreviewCallback {
+    inner: Arc<dyn Fn(Vec<Arc<dyn SkimItem>>) -> Vec<AnsiString<'static>> + Send + Sync + 'static>,
+}
+
+impl<F> From<F> for PreviewCallback
+where
+    F: Fn(Vec<Arc<dyn SkimItem>>) -> Vec<AnsiString<'static>> + Send + Sync + 'static,
+{
+    fn from(func: F) -> Self {
+        Self { inner: Arc::new(func) }
+    }
+}
+
+impl std::ops::Deref for PreviewCallback {
+    type Target = dyn Fn(Vec<Arc<dyn SkimItem>>) -> Vec<AnsiString<'static>> + Send + Sync + 'static;
+
+    fn deref(&self) -> &Self::Target {
+        &*self.inner
+    }
+}
+
 pub struct Previewer {
     tx_preview: Sender<PreviewEvent>,
     content_lines: Arc<SpinLock<Vec<AnsiString<'static>>>>,
@@ -37,14 +60,22 @@ pub struct Previewer {
     prev_cmd_query: Option<String>,
     prev_num_selected: usize,
 
-    preview_cmd: Option<String>,
+    preview_source: PreviewSource,
     preview_offset: String, // e.g. +SCROLL-OFFSET
     delimiter: Regex,
     thread_previewer: Option<JoinHandle<()>>,
 }
 
+/// Source for the Preview window.
+#[non_exhaustive]
+pub enum PreviewSource {
+    Empty,
+    Command(String),
+    Callback(PreviewCallback),
+}
+
 impl Previewer {
-    pub fn new<C>(preview_cmd: Option<String>, callback: C) -> Self
+    pub fn new<C>(source: PreviewSource, callback: C) -> Self
     where
         C: Fn() + Send + Sync + 'static,
     {
@@ -81,19 +112,16 @@ impl Previewer {
         Self {
             tx_preview,
             content_lines,
-
             width,
             height,
             hscroll_offset,
             vscroll_offset,
             wrap: false,
-
             prev_item: None,
             prev_query: None,
             prev_cmd_query: None,
             prev_num_selected: 0,
-
-            preview_cmd,
+            preview_source: source,
             preview_offset: "".to_string(),
             delimiter: Regex::new(DELIMITER_STR).unwrap(),
             thread_previewer: Some(thread_previewer),
@@ -219,19 +247,26 @@ impl Previewer {
                         PreviewEvent::PreviewCommand(preview_command, pos)
                     }
                 }
-                (ItemPreview::Global, _) => {
-                    let cmd = self.preview_cmd.clone().expect("previewer: not provided");
-                    if depends_on_items(&cmd) && self.prev_item.is_none() {
-                        debug!("the command for preview refers to items and currently there is no item");
-                        debug!("command to execute: [{}]", cmd);
-                        PreviewEvent::PreviewPlainText("no item matched".to_string(), Default::default())
-                    } else {
-                        let cmd = inject_command(&cmd, inject_context).to_string();
-                        let pos = self.eval_scroll_offset(inject_context);
-                        let preview_command = PreviewCommand { cmd, columns, lines };
-                        PreviewEvent::PreviewCommand(preview_command, pos)
+                (ItemPreview::Global, _) => match &self.preview_source {
+                    PreviewSource::Command(cmd) => {
+                        if depends_on_items(cmd) && self.prev_item.is_none() {
+                            debug!("the command for preview refers to items and currently there is no item");
+                            debug!("command to execute: [{}]", cmd);
+                            PreviewEvent::PreviewPlainText("no item matched".to_string(), Default::default())
+                        } else {
+                            let cmd = inject_command(cmd, inject_context).to_string();
+                            let pos = self.eval_scroll_offset(inject_context);
+                            let preview_command = PreviewCommand { cmd, columns, lines };
+                            PreviewEvent::PreviewCommand(preview_command, pos)
+                        }
                     }
-                }
+                    PreviewSource::Callback(cb) => {
+                        let pos = self.eval_scroll_offset(inject_context);
+                        let cb = cb.clone();
+                        PreviewEvent::PreviewCallback(Box::new(move || cb(selections)), pos)
+                    }
+                    PreviewSource::Empty => PreviewEvent::Noop,
+                },
             },
             None => PreviewEvent::Noop,
         };
@@ -403,8 +438,12 @@ pub struct PreviewCommand {
     pub columns: usize,
 }
 
-#[derive(Debug)]
+// #[derive(Debug)]
 enum PreviewEvent {
+    PreviewCallback(
+        Box<dyn FnOnce() -> Vec<AnsiString<'static>> + Send + Sync + 'static>,
+        PreviewPosition,
+    ),
     PreviewCommand(PreviewCommand, PreviewPosition),
     PreviewPlainText(String, PreviewPosition),
     PreviewAnsiText(String, PreviewPosition),
@@ -490,6 +529,7 @@ where
                     }
                 }
             }
+            PreviewEvent::PreviewCallback(cb, pos) => callback(cb(), pos),
             PreviewEvent::PreviewPlainText(text, pos) => {
                 callback(text.lines().map(|line| line.to_string().into()).collect(), pos);
             }
